
#include <cassert>
#include <cstdio>
#include <cstring>
#include <functional>

#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

template<typename...Args> int
spawn(std::function<void()> fix_child, const char *prog, Args...args)
{
  pid_t pid = fork();
  if (pid == 0) {
    fix_child();
    execlp(prog, prog, args..., nullptr); // Note prog is repeated
    perror(prog);
    _exit(1);			// Note use of _exit
  }
  if (pid == -1)
    std::perror("fork");
  return pid;
}

template<typename...Args> int
spawn(const char *prog, Args...args)
{
  return spawn([]{}, prog, args...);
}

void
join(pid_t pid)
{
  if (int status; waitpid(pid, &status, 0) == -1)
    perror("waitpid");
  else if (status)
    std::fprintf(stderr, "pid %d exited with status 0x%x\n", pid, status);
}

int
main()
{
  // equivalent to: gpg ... 2> /dev/null
  join(spawn([]{
    close(2);
    open("/dev/null", O_WRONLY);
  }, "gpg", "--pinentry-mode=loopback", "-d", "secret.gpg"));

  // equivalent to: gpg-connect-agent ... < /dev/null > /dev/null
  join(spawn([]{
    close(0);
    open("/dev/null", O_RDONLY);
    close(1);
    open("/dev/null", O_WRONLY);
  }, "gpg-connect-agent", "-q", "killagent"));
}

// Annoyance: Have to type password
