
#include <cassert>
#include <cstdio>
#include <cstring>
#include <functional>
#include <string>

#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

template<typename...Args> int
spawn(std::function<void()> fix_child, const char *prog, Args...args)
{
  pid_t pid = fork();
  if (pid == 0) {
    fix_child();
    execlp(prog, prog, args..., nullptr); // Note prog is repeated
    perror(prog);
    _exit(1);			// Note use of _exit
  }
  if (pid == -1)
    std::perror("fork");
  return pid;
}

template<typename...Args> int
spawn(const char *prog, Args...args)
{
  return spawn([]{}, prog, args...);
}

void
join(pid_t pid)
{
  if (int status; waitpid(pid, &status, 0) == -1)
    perror("waitpid");
  else if (status)
    std::fprintf(stderr, "pid %d exited with status 0x%x\n", pid, status);
}

int
main()
{
  int devnull = open("/dev/null", O_RDWR);
  if (devnull == -1) {
    perror("/dev/null");
    exit(1);
  }
  fcntl(devnull, F_SETFD, 1);	// Set close on exec flag

  int pipefds[2];
  if (pipe(pipefds) == -1) {
    perror("pipe");
    exit(1);
  }

  pid_t pid = spawn([devnull,pipefds]{
    close(pipefds[1]);
    dup2(devnull, 2);
  }, "gpg", "--passphrase-fd", std::to_string(pipefds[0]).c_str(),
      "--pinentry-mode=loopback", "-d", "secret.gpg");

  close(pipefds[0]);
  write(pipefds[1], "test", 4);
  close(pipefds[1]);
  join(pid);

  join(spawn([devnull]{
    dup2(devnull, 0);
    dup2(devnull, 1);
  }, "gpg-connect-agent", "-q", "killagent"));
}
